/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package bufferings.ktr.i18n.client;

import bufferings.ktr.i18n.server.KtrI18NCreator;

import com.google.gwt.i18n.client.Constants;
import com.google.gwt.i18n.client.Messages;

/**
 * KtrI18N(Kotori I18N) is the seamless GWT I18N module which works on both the
 * client and the server.
 * 
 * @author bufferings[at]gmail.com
 */
public class KtrI18N {

  /**
   * Creates an implementation instance of {@code Constants}.
   * <p>
   * On the client, the GWT locale is used. On the server, the default locale is
   * used.
   * </p>
   * <p>
   * Notice: {@code ConstantsWithLookup} is not supported.
   * </p>
   * 
   * @param <T>
   *          a type of return class, which must be typecast to the requested
   *          class.
   * @param constantsClassLiteral
   *          a class literal specifying the base class to be instantiated
   * @return the new implementation instance
   */
  public static <T> T createConstants(
      Class<? extends Constants> constantsClassLiteral) {
    return createConstants(constantsClassLiteral, "", "");
  }

  /**
   * Creates an implementation instance of {@code Constants} with the locale.
   * <p>
   * On the client, the GWT locale is used. On the server, the specified locale
   * is used.
   * </p>
   * <p>
   * Notice: {@code ConstantsWithLookup} is not supported.
   * </p>
   * 
   * @param <T>
   *          a type of return class, which must be typecast to the requested
   *          class.
   * @param constantsClassLiteral
   *          a class literal specifying the base class to be instantiated
   * @param language
   *          lowercase two-letter ISO-639 code.
   * @param country
   *          uppercase two-letter ISO-3166 code.
   * @return the new implementation instance
   */
  public static <T> T createConstants(
      Class<? extends Constants> constantsClassLiteral, String language,
      String country) {
    return getKtrI18NCreator().create(constantsClassLiteral, language, country);
  }

  /**
   * Creates an implementation instance of {@code Messages}
   * <p>
   * On the client, the GWT locale is used. On the server, the default locale is
   * used.
   * </p>
   * 
   * @param <T>
   *          a type of return class, which must be typecast to the requested
   *          class.
   * @param messagesClassLiteral
   *          a class literal specifying the base class to be instantiated
   * @return the new implementation instance
   */
  public static <T> T createMessages(
      Class<? extends Messages> messagesClassLiteral) {
    return createMessages(messagesClassLiteral, "", "");
  }

  /**
   * Creates an implementation instance of {@code Messages} with the locale.
   * <p>
   * On the client, the GWT locale is used. On the server, the specified locale
   * is used.
   * </p>
   * 
   * @param <T>
   *          a type of return class, which must be typecast to the requested
   *          class.
   * @param messagesClassLiteral
   *          a class literal specifying the base class to be instantiated
   * @param language
   *          lowercase two-letter ISO-639 code.
   * @param country
   *          uppercase two-letter ISO-3166 code.
   * @return the new implementation instance
   */
  public static <T> T createMessages(
      Class<? extends Messages> messagesClassLiteral, String language,
      String country) {
    return getKtrI18NCreator().create(messagesClassLiteral, language, country);
  }

  /**
   * Gets a KtrI18Ncreator instance.
   * <p>
   * Separate implementations of the KtrI18NCreator are used on the client and
   * the server, using the super-source.
   * </p>
   * 
   * @return a new KtrI18Ncreator instance
   */
  protected static KtrI18NCreator getKtrI18NCreator() {
    return new KtrI18NCreator();
  }

}
