/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package bufferings.ktr.i18n.server;

import bufferings.ktr.i18n.client.KtrI18NException;

import com.google.gwt.i18n.client.Constants;
import com.google.gwt.i18n.client.ConstantsWithLookup;
import com.google.gwt.i18n.client.Messages;

/**
 * The KtrI18NCreator for the server.
 * <p>
 * The KtrI18NCreator source for the client is in the super-source.
 * </p>
 * 
 * @author bufferings[at]gmail.com
 */
public class KtrI18NCreator {

  /**
   * Instantiates a class of the GWT I18N interface implementation for the
   * server.
   * <p>
   * The {@code classLiteral} must be typecast to the {@link Constants} or
   * {@link Messages}. {@link ConstantsWithLookup} is not supported.
   * </p>
   * 
   * @param <T>
   *          a class type of your I18N interface
   * @param classLiteral
   *          a class literal specifying the base class to be instantiated
   * @param language
   *          lowercase two-letter ISO-639 code.
   * @param country
   *          uppercase two-letter ISO-3166 code.
   * @return the new instance of the classLiteral interface implementation.
   * @throws KtrI18NException
   *           if the classLiteral is a typecast to the unsupported class.
   * @throws KtrI18NException
   *           if the instantiation fails
   */
  public <T> T create(Class<?> classLiteral, String language, String country) {
    GeneratorTemplate generator;

    if (ConstantsWithLookup.class.isAssignableFrom(classLiteral)) {
      throw new KtrI18NException(
        "The classLiteral is not supported by KtrI18N(classLiteral="
          + classLiteral.getName()
          + ").");
    } else if (Constants.class.isAssignableFrom(classLiteral)) {
      generator = new ConstantsGenerator();
    } else if (Messages.class.isAssignableFrom(classLiteral)) {
      generator = new MessagesGenerator();
    } else {
      throw new KtrI18NException(
        "The classLiteral is not supported by KtrI18N(classLiteral="
          + classLiteral.getName()
          + ").");
    }

    String interfaceName = classLiteral.getName();
    return generator.generate(interfaceName, language, country);
  }
}
