/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package bufferings.ktr.i18n.server;

import bufferings.ktr.i18n.client.KtrI18NException;

import com.google.gwt.i18n.client.Constants;
import com.google.gwt.i18n.client.ConstantsWithLookup;
import com.google.gwt.i18n.client.Messages;

/**
 * Super-source of KtrI18NCreator for the GWT client.
 * <p>
 * This super-source is not used. In the compile time, the KtrI18NPlugin(Kotor
 * I18N Plugin) replace {@code KtrI18N#createXXX()} in your code with {@code
 * GWT#create(Class)} using the super-source.
 * </p>
 * <p>
 * The reason why I created KtrI18NPlugin is the limitation that {@code
 * GWT#create(Class)} must be called with a class literal. If GWT supports <a
 * href="http://code.google.com/p/google-web-toolkit/issues/detail?id=2243">
 * Issue 2243</a> or <a href="http://groups.google.co.jp/group/google-web-toolkit-contributors/browse_thread/thread/fe37fcbc10bc4032"
 * > Relaxing constraints on GWT.create()</a>, I can throw the KtrI18NPlugin
 * away with pleasure and we can use the commented out code in this source.
 * </p>
 * <p>
 * I hope GWT to support that.
 * </p>
 * 
 * @author bufferings[at]gmail.com
 * @see com.google.gwt.core.client.GWT
 */
public class KtrI18NCreator {

  /**
   * Instantiates a class of the GWT I18N interface implementation.
   * <p>
   * The {@code classLiteral} must be a typecast to the {@link Constants},
   * {@link Messages} or {@link ConstantsWithLookup}.
   * </p>
   * 
   * @param <T>
   *          a class type of your I18N interface
   * @param classLiteral
   *          a class literal specifying the base class to be instantiated
   * @param language
   *          not for the GWT client, for the server
   * @param country
   *          not for the GWT client, for the server
   * @return the new instance of the classLiteral interface implementation.
   * @throws KtrI18NException
   *           if the classLiteral is not a typecast to the one of supported
   *           types.
   */
  public <T> T create(Class<?> classLiteral, String language, String country) {
    throw new KtrI18NException(
      "Please use the KtrI18NPlugin(Kotori I18N Plugin).");

    // if (ConstantsWithLookup.class.isAssignableFrom(classLiteral)) {
    // throw new KtrI18NException(
    // "The classLiteral is not supported by KtrI18N(classLiteral="
    // + classLiteral.getName()
    // + ").");
    // } else if (Constants.class.isAssignableFrom(classLiteral)) {
    // return GWT.create(classLiteral);
    // } else if (Messages.class.isAssignableFrom(classLiteral)) {
    // return GWT.create(classLiteral);
    // } else {
    // throw new KtrI18NException(
    // "The classLiteral is not supported by KtrI18N(classLiteral="
    // + classLiteral.getName()
    // + ").");
    // }
  }

}
